// 应用的入口文件，在项目根目录，统一命名为main.rs
// 库的入口文件统一命名为lib.rs

// 引用模块
mod test;
mod keygen;

// 将其他模块的成员声明为当前模块直接可用的，等同于java的静态引用
// 若省略这句，下面不可以直接使用Random结构体，而需要加上模块名，即test::Random
use self::test::Random;

use std::thread;
use std::sync::mpsc;
use std::collections::HashMap;
use chrono::prelude::*;

use std::io::{stdout, BufWriter};
use ferris_says::say;

fn main() {
    let stdout = stdout();
    let out = b"Hello fellow Rustaceans!";
    let width = 24;

    let mut writer = BufWriter::new(stdout.lock());
    say(out, width, &mut writer).unwrap(); // unwrap为rust预定义错误处理模式，正常时返回OK值，错误时panic！

    let rd = Random { seed: 100 };
    println!("a random number:{}", rd.next());
    let id = keygen::next_id();
    println!("key from keygen:{}", id);
    println!();
    test_next_id();
}

fn test_next_id() {
    let id_size = 100000;

    let start_time = Utc::now().timestamp_nanos();
    let (tx, rx) = mpsc::channel();


    for _ in 0..id_size {
        let tx = tx.clone();
        thread::spawn(move || {
            let id = keygen::next_id();
            tx.send(id).unwrap()
        });
    };

    let (tx2, rx2) = mpsc::channel();
    let tx2 = tx2.clone();
    let handle = thread::spawn(move || {
        let mut m: HashMap<u64, i32> = HashMap::new();
        let mut i = id_size;
        loop {
            let id = rx.recv().unwrap();
            //let mut wrap = wrap.lock().unwrap();
            if m.contains_key(&id) {
                let v = *(m.get_mut(&id).unwrap());
                m.insert(id, v + 1);
            } else {
                m.insert(id, 1);
            }
            i -= 1;
            if i == 0 {
                break;
            }
        }
        tx2.send(m).unwrap();
    });

    handle.join().expect("The thread being joined has panicked");

    let m = rx2.recv().unwrap();

    let end_time = Utc::now().timestamp_nanos();
    let used_time = end_time - start_time;
    println!("Id size: {}\nTotal seconds:{}\nAvg nanos per id:{}", m.len(), used_time / 1000000000, (used_time as i64) / (id_size as i64));

    let mut bad_size = 0;
    for (_, v) in m.iter() {
        if *v > 1 {
            bad_size += 1;
        }
    }

    if bad_size > 0 {
        println!("Not good")
    }
}

